@@ -2,8 +2,6 @@
 from contextlib import contextmanager
 import logging
 import io
-import os
-import platform
 import subprocess
 import sys
 from datetime import timedelta
@@ -15,6 +13,7 @@ from .sklearn_shim import Pipeline
 import tqdm
 
 from .constants import *
+from .ffmpeg_utils import ffmpeg_bin_path, subprocess_args
 from .subtitle_parser import make_subtitle_parser
 from .subtitle_transformers import SubtitleScaler
 
@@ -22,73 +21,6 @@ logging.basicConfig(level=logging.INFO)
 logger = logging.getLogger(__name__)
 
 
-# ref: https://github.com/pyinstaller/pyinstaller/wiki/Recipe-subprocess
-# Create a set of arguments which make a ``subprocess.Popen`` (and
-# variants) call work with or without Pyinstaller, ``--noconsole`` or
-# not, on Windows and Linux. Typical use::
-#
-#   subprocess.call(['program_to_run', 'arg_1'], **subprocess_args())
-#
-# When calling ``check_output``::
-#
-#   subprocess.check_output(['program_to_run', 'arg_1'],
-#                           **subprocess_args(False))
-def _subprocess_args(include_stdout=True):
-    # The following is true only on Windows.
-    if hasattr(subprocess, 'STARTUPINFO'):
-        # On Windows, subprocess calls will pop up a command window by default
-        # when run from Pyinstaller with the ``--noconsole`` option. Avoid this
-        # distraction.
-        si = subprocess.STARTUPINFO()
-        si.dwFlags |= subprocess.STARTF_USESHOWWINDOW
-        # Windows doesn't search the path by default. Pass it an environment so
-        # it will.
-        env = os.environ
-    else:
-        si = None
-        env = None
-
-    # ``subprocess.check_output`` doesn't allow specifying ``stdout``::
-    #
-    #   Traceback (most recent call last):
-    #     File "test_subprocess.py", line 58, in <module>
-    #       **subprocess_args(stdout=None))
-    #     File "C:\Python27\lib\subprocess.py", line 567, in check_output
-    #       raise ValueError('stdout argument not allowed, it will be overridden.')
-    #   ValueError: stdout argument not allowed, it will be overridden.
-    #
-    # So, add it only if it's needed.
-    if include_stdout:
-        ret = {'stdout': subprocess.PIPE}
-    else:
-        ret = {}
-
-    # On Windows, running this from the binary produced by Pyinstaller
-    # with the ``--noconsole`` option requires redirecting everything
-    # (stdin, stdout, stderr) to avoid an OSError exception
-    # "[Error 6] the handle is invalid."
-    ret.update({'stdin': subprocess.PIPE,
-                'stderr': subprocess.PIPE,
-                'startupinfo': si,
-                'env': env})
-    return ret
-
-
-def _ffmpeg_bin_path(bin_name, gui_mode, ffmpeg_resources_path=None):
-    if platform.system() == 'Windows':
-        bin_name = '{}.exe'.format(bin_name)
-    if ffmpeg_resources_path is not None:
-        return os.path.join(ffmpeg_resources_path, bin_name)
-    try:
-        resource_path = os.environ[SUBSYNC_RESOURCES_ENV_MAGIC]
-        if len(resource_path) > 0:
-            return os.path.join(resource_path, 'ffmpeg-bin', bin_name)
-    except KeyError as e:
-        if gui_mode:
-            logger.info("Couldn't find resource path; falling back to searching system path")
-    return bin_name
-
-
 def make_subtitle_speech_pipeline(
         fmt='srt',
         encoding=DEFAULT_ENCODING,
@@ -212,7 +144,7 @@ class VideoSpeechTransformer(TransformerMixin):
         else:
             streams_to_try = [self.ref_stream]
         for stream in streams_to_try:
-            ffmpeg_args = [_ffmpeg_bin_path('ffmpeg', self.gui_mode, ffmpeg_resources_path=self.ffmpeg_path)]
+            ffmpeg_args = [ffmpeg_bin_path('ffmpeg', self.gui_mode, ffmpeg_resources_path=self.ffmpeg_path)]
             ffmpeg_args.extend([
                 '-loglevel', 'fatal',
                 '-nostdin',
@@ -221,7 +153,7 @@ class VideoSpeechTransformer(TransformerMixin):
                 '-f', 'srt',
                 '-'
             ])
-            process = subprocess.Popen(ffmpeg_args, **_subprocess_args(include_stdout=True))
+            process = subprocess.Popen(ffmpeg_args, **subprocess_args(include_stdout=True))
             output = io.BytesIO(process.communicate()[0])
             if process.returncode != 0:
                 break
@@ -245,7 +177,7 @@ class VideoSpeechTransformer(TransformerMixin):
                 logger.info(e)
         try:
             total_duration = float(ffmpeg.probe(
-                fname, cmd=_ffmpeg_bin_path('ffprobe', self.gui_mode, ffmpeg_resources_path=self.ffmpeg_path)
+                fname, cmd=ffmpeg_bin_path('ffprobe', self.gui_mode, ffmpeg_resources_path=self.ffmpeg_path)
             )['format']['duration']) - self.start_seconds
         except Exception as e:
             logger.warning(e)
@@ -257,7 +189,7 @@ class VideoSpeechTransformer(TransformerMixin):
         else:
             raise ValueError('unknown vad: %s' % self.vad)
         media_bstring = []
-        ffmpeg_args = [_ffmpeg_bin_path('ffmpeg', self.gui_mode, ffmpeg_resources_path=self.ffmpeg_path)]
+        ffmpeg_args = [ffmpeg_bin_path('ffmpeg', self.gui_mode, ffmpeg_resources_path=self.ffmpeg_path)]
         if self.start_seconds > 0:
             ffmpeg_args.extend([
                 '-ss', str(timedelta(seconds=self.start_seconds)),
@@ -276,7 +208,7 @@ class VideoSpeechTransformer(TransformerMixin):
             '-ar', str(self.frame_rate),
             '-'
         ])
-        process = subprocess.Popen(ffmpeg_args, **_subprocess_args(include_stdout=True))
+        process = subprocess.Popen(ffmpeg_args, **subprocess_args(include_stdout=True))
         bytes_per_frame = 2
         frames_per_window = bytes_per_frame * self.frame_rate // self.sample_rate
         windows_per_buffer = 10000
@@ -301,7 +233,7 @@ class VideoSpeechTransformer(TransformerMixin):
                     if not in_bytes:
                         break
                     newstuff = len(in_bytes) / float(bytes_per_frame) / self.frame_rate
-                    if simple_progress + newstuff > total_duration:
+                    if total_duration is not None and simple_progress + newstuff > total_duration:
                         newstuff = total_duration - simple_progress
                     simple_progress += newstuff
                     pbar.update(newstuff)
